/*
 * Copyright (c) 2003-2006 Erez Zadok
 * Copyright (c) 2003-2006 Stony Brook University
 * Copyright (c) 2003-2006 The Research Foundation of State University of New York
 * Copyright (c) 2004-2006 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2006 Gopala Surya
 */

#ifndef __DEBUG_H
#define __DEBUG_H

#include <stdio.h>

//ERROR SETTINGS
#define ERROR_MESSAGES 1
#define WARNING_MESSAGES 1

//DEBUG SETTINGS
#define DEBUG_LOCATION 0
#define DEBUG_VALUES 1

#if DEBUG_LOCATION
#define print_entry_location() fprintf(stderr,"DEBUG: ENTERING: %s, %s:%d\n",__FUNCTION__,__FILE__,__LINE__)
#define print_exit_location() fprintf(stderr,"DEBUG: EXITING: %s, %s:%d\n",__FUNCTION__,__FILE__,__LINE__)
#else //#if DEBUG_LOCATION
#define print_entry_location()
#define print_exit_location()
#endif //#if DEBUG_LOCATION

#if DEBUG_VALUES
#define print_debug_int(VAR,VAL) fprintf(stderr,"DEBUG: %s() - %s = %d\n",__FUNCTION__,VAR,VAL)
#define print_debug_addr(VAR,VAL) fprintf(stderr,"DEBUG: %s() - %s = %x\n",__FUNCTION__,VAR,(int)VAL)
#define print_debug_string(VAR,STR) fprintf(stderr,"DEBUG: %s() - %s = %s\n",__FUNCTION__,VAR,STR)
#define print_debug_location(LOC) fprintf(stderr,"DEBUG: %s() - %s\n",__FUNCTION__,LOC)
#define print_debug_message(FORMAT,ARGS...) {\
             fprintf(stderr,"DEBUG: %s() - ",__FUNCTION__); \
             fprintf(stderr,FORMAT,##ARGS); \
             fprintf(stderr,"\n"); }
#else //#if DEBUG_VALUES
#define print_debug_int(VAR,VAL)
#define print_debug_addr(VAR,VAL)
#define print_debug_string(VAR,VAL)
#define print_debug_location(LOC)
#define print_debug_message(FORMAT,ARGS...)
#endif //#if DEBUG_VALUES

#if ERROR_MESSAGES
#define print_error(STR) fprintf(stderr,"ERROR: %s (%s, %s:%d)\n",STR,__FUNCTION__,__FILE__,__LINE__)
#define print_error_message(FORMAT,ARGS...){\
             fprintf(stderr,"ERROR: (%s, %s:%d)",__FUNCTION__,__FILE__,__LINE__); \
             fprintf(stderr,FORMAT,##ARGS); \
             fprintf(stderr,"\n"); }
#define print_error_nomem() print_error("Malloc Failed: Out of Memory")
#else
#define print_error(STR)
#define print_error_message(FORMAT,ARGS...)
#define print_error_nomem()
#endif

#if WARNING_MESSAGES
#define print_warning_message(FORMAT,ARGS...) {\
             fprintf(stderr,"WARNING: %s() - ",__FUNCTION__); \
             fprintf(stderr,FORMAT,##ARGS); \
             fprintf(stderr,"\n"); }
#else
#define print_warning_message(FORMAT,ARGS...)
#endif


#endif //#ifndef __DEBUG_H
