/*
 * Copyright (c) 2003-2006 Erez Zadok
 * Copyright (c) 2003-2006 Stony Brook University
 * Copyright (c) 2003-2006 The Research Foundation of State University of New York
 * Copyright (c) 2004-2006 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2006 Gopala Surya
 */


#ifndef __IDRIVER_H
#define __IDRIVER_H

#include <errno.h>
#include <stdio.h>
#include "tracereader.h"

enum parse_state {
	BEGIN,
	IN_MSG,
	IN_MSG_LEN,
	IN_MSG_SEQ_NUM,
	IN_MSG_READ,
	IN_ARG_TV,
	IN_ARG_TLV,
	IN_ARG_TLV_GETLEN
};

#define ERR_SIZE 256
struct parser {
    int fd; /* file descriptor for reading */
    char buf[4096]; /* this is a buffer for the main parser */
    char err[ERR_SIZE]; /* error buffer */
    int msg_num;
    int pre_or_post;
    op_t current_msg_type;
    field_t current_arg_type;
    void* context; /* context carried through a message */
    int sequence_number; //Binds PRE/POST OP to the same context
    enum parse_state state;
    struct idriver *idriver;
    int id; //ID of the pre/post operation to keep track of the context

};


struct input_ops;
struct idriver {
	struct idriver *next_driver;
	struct input_ops *ops;
};


struct input_ops {
	// returns ERR_OK on success, error code otherwise
	int (*init)(struct parser *p, void *params, struct idriver *next);
	// returns # of bytes read, negative error code otherwise
	int (*read)(struct parser *p, char *buf, int len);
	void (*release)();
};


// pass file name for file_idriver initialization
extern struct idriver *file_idriver;

// pass buf_size as cksum_idriver initialization
extern struct idriver *cksum_idriver;

extern struct idriver *uncompress_idriver;


extern struct idriver *decrypt_idriver;
#endif
